//+------------------------------------------------------------------------------------+
#define     _NAME_      "DayBreakTrail"
#define     _TITLE_     "DayBreakTrail (v1.013) - MetaTrader 4 Expert Advisor"
#property   version     "1.013"
#property   copyright   "Copyright  2014, Fernando M. I. Carreiro"
#property   link        "mailto:fmi@carreiro.co.pt?subject=DayBreakTrail.v1.013"
#property   description _TITLE_
#property   strict
//+------------------------------------------------------------------------------------+


//+------------------------------------------------------------------------------------+
// Includes and Library References
//+------------------------------------------------------------------------------------+

#include <stdlib.mqh>
#include <stderror.mqh>


//+------------------------------------------------------------------------------------+
// Input Parameters (External Variables)
//+------------------------------------------------------------------------------------+

//---- Define Trading Signal States
enum
   ENUM_TRADING_SIGNAL     // Define Trade Signaling
      {
            _UP_LONG_      =  +1
         ,  _SIDE_OUT_     =   0
         ,  _DOWN_SHORT_   =  -1
      };
      
      
//--- Title Identification
input string
   strTitle                   = _TITLE_
;

//--- Expert Advisor Settings
input string
   strPrimarySettings         = "--- Expert Advisor Settings ---"
;

   //---- Define DayBreak Parameters and Variables
   input string
      strDayBreakSettings     = "--- Daily Breakout Settings (H1 Chart) ---"
   ;
   extern int
      intExpirePendingDays    = 1            // Expiration Time for Pending Orders (days)
   ,  intExpireOpenHours      = 4            // Expiration Time for Open Orders (hours)
   ,  intOpenHour             = -1           // Opening of Daily Session (hour, -1 Auto)
   ;
   extern double
      dblHighOffsetPips       = 0.0          // Offset on High Break (pips)
   ,  dblLowOffsetPips        = 0.0          // Offset on Low Break (pips)
   ;
   string
      nameExpirePendingDays   = "ExpirePendingDays (int)"
   ,  nameExpireOpenHours     = "ExpireOpenHours (int)"
   ,  nameOpenHour            = "OpenHour (int)"
   ,  nameHighOffsetPips      = "HighOffsetPips (double)"
   ,  nameLowOffsetPips       = "LowOffsetPips (double)"
   ;
   int
      intDayArraySize         = EMPTY        // Size of Daily Highs and Lows arrays
   ,  intDayHighIndex         = EMPTY        // Index in to Daily Highs array
   ,  intDayLowIndex          = EMPTY        // Index in to Daily Lows array
   ,  intExpireOpenSeconds                   // Expiration Time for Open Orders (seconds)
   ;
   double
      dblHighOffsetDelta      // Offset on High Break (delta)
   ,  dblLowOffsetDelta       // Offset on Low Break (delta)
   ,  dblDailyHighs[]         // Previous Daily Highs for number of Pending Days
   ,  dblDailyLows[]          // Previous Daily Low for number of Pending Days
   ;
   ENUM_TRADING_SIGNAL
      enumSignalDayBreak      // Daily Breakout Signal
   ;


   //---- Define Spread Management Parameters and Variables
   input string
      strSpreadSettings       = "--- Spread Management Settings ---"
   ;
   extern double
      dblSpreadMaximumPips    = 3.0    // Maximum Spread Allowed  (Pips)
   ;
   string
      nameSpreadMaximumPips   = "SpreadMaximumPips (double)"
   ;
   double
      dblSpreadMaximumDelta   // Maximum Spread Allowed  (Delta)
   ,  dblSpreadDelta          // Current Spread Value    (Delta)
   ,  dblSpreadPips           // Current Spread Value    (Pips)
   ;
   bool
      boolSignalSpread        // Signal when Spread is below threshold
   ;  
   
   
   //---- Define Stops Management Parameters and Variables
   input string
      strStopsSettings        = "--- Stops Management Settings ---"
   ;
   extern double
      dblStopMinimumPips      = 0.0       // Minimum Size of Stops   (Pips)
   ,  dblSoftTakeProfitPips   = 1000      // Soft Take-Profit        (Pips)
   ,  dblSoftStopLossPips     = 5         // Soft Stop-Loss          (Pips)
   ,  dblSoftTrailStopPips    = 5         // Soft Trailing-Stop      (Pips)
   ,  dblSoftTrailStepPips    = 5         // Soft Trailing-Step      (Pips)
   ;
   string
      nameStopMinimumPips     = "StopMinimumPips (double)"
   ,  nameSoftTakeProfitPips  = "SoftTakeProfitPips (double)"
   ,  nameSoftStopLossPips    = "SoftStopLossPips (double)"
   ,  nameSoftTrailStopPips   = "SoftTrailStopPips (double)"
   ,  nameSoftTrailStepPips   = "SoftTrailStepPips (double)"
   ;
   double
      dblStopMinimumDelta     // Minimum Size of Stops  (Delta)
   ,  dblSoftTakeProfitDelta  // Soft TakeProfit        (Delta)
   ,  dblSoftStopLossDelta    // Soft Stop-Loss         (Delta)
   ,  dblSoftTrailStopDelta   // Soft Trailing-Stop     (Delta)
   ,  dblSoftTrailStepDelta   // Soft Trailing-Step     (Delta)
   ;
   
   
   //---- Define Lots Management Parameters and Variables
   input string
      strLotsSettings         = "--- Lots Management Settings ---"
   ;
   extern double
      dblLotsMinimum          = 0.01      // Minimum Size of Lots per Order
   ,  dblLotsMaximum          = 100.00    // Maximum Size of Lots per Order
   ,  dblLotsStep             = 0.01      // Minimum Size Interval of Lots
   ;
   string
      nameLotsMinimum         = "LotsMinimum (double)"
   ,  nameLotsMaximum         = "LotsMaximum (double)"
   ,  nameLotsStep            = "LotsStep (double)"
   ;

   
   //---- Define Slippage Management Parameters and Variables
   input string
      strSlippageSettings     = "--- Slippage Management Settings ---"
   ;
   extern double
      dblSlippageEnterPips    = 2.0    // Slippage when Entering Trade  (Pips)
   ,  dblSlippageExitPips     = 2.0    // Slippage when Exiting Trade   (Pips)
   ;
   string
      nameSlippageEnterPips   = "SlippageEnterPips (double)"
   ,  nameSlippageExitPips    = "SlippageExitPips (double)"
   ;     
   int
      intSlippageEnterPoints  // Slippage when Entering Trade  (Points)
   ,  intSlippageExitPoints   // Slippage when Exiting Trade   (Points)
   ;


   //---- Define Money and Risk Management Parameters and Variables
   input string
      strMoneyRiskSettings    = "--- Money & Risk Management Settings ---"
   ;
   extern double
      dblRiskMaximum          = 1.00      // Maximum Risk per Order     (Percentage)
   ,  dblMarginMaximum        = 5.00      // Maximum Margin per Order   (Percentage)
   ,  dblBalanceMinimum       = 0.00      // Minimum Balance Requirement / Protection
   ;
   string
      nameRiskMaximum         = "RiskMaximum (double)"
   ,  nameMarginMaximum       = "MarginMaximum (double)"
   ,  nameBalanceMinimum      = "BalanceMinimum (double)"
   ;


   //---- Define Magic Numbers Parameters and Variables
   //    (upper 4 nibbles only, lower used for differentiation)
   extern string
      strMagicNumberSettings  = "--- Magic Numbers Settings (Major: 0-4095, Minor: 0-15) ---"
   ;
   extern int
      intMagicMajor           = 0x401  // Major Magic Number (3 nibbles)
   ,  intMagicMinor           = 0x1    // Minor Magic Number (1 nibble)
   ;
   string
      nameMagicMajor          = "MagicMajor (int)"
   ,  nameMagicMinor          = "MagicMinor (int)"
   ;
   int
      intMagicMask            // Magic Number Mask for Strategy
   ,  intMagicFirst           // Magic Number (First/Minimum)
   ,  intMagicLast            // Magic Number (Last/Maximum)
   ,  intMagicPrimary         // Magic Number for Primary Strategy
   ;


   //---- Define Global Variable Parameter Update Interval
   input int
      intParamUpdateMins      = 5      // Update Interval for Global Parameters
   ;

   //---- Define Parameters for Order Retry
   input int
      intRetryCount           = 10     // Order Update Retry Count
   ;
   input double
      dblSleepTimeMean        = 8.0    // Average Sleep Time
   ,  dblSleepTimeMax         = 25.0   // Maximum Sleep Time
   ;


//+------------------------------------------------------------------------------------+
// Global Scope Enumerations and Variables
//+------------------------------------------------------------------------------------+

//---- Define Trading States Enumeration and Variables
enum
   ENUM_TRADING_STATE      // Define Trading States
      {
            _TRADING_FULL_ =  +1
         ,  _TRADING_EXIT_ =   0
         ,  _TRADING_NONE_ =  -1
      };
ENUM_TRADING_STATE
   enumTradingState        = _TRADING_FULL_  // Allow Trading on this EA
,  enumTradingStateAll     = _TRADING_FULL_  // Allow Trading on All EA's
;
string
   nameTradingStateAll     = "TradingState (int)"
,  nameTradingState        = "TradingState (int)"
;




//+------------------------------------------------------------------------------------+
// Prepare Global Context Variables Names with Chart Prefix
//+------------------------------------------------------------------------------------+
string
   namePrefix  = ""
,  nameSymbol  = "Unknown Symbol"
,  namePeriod  = "Unknown Period Name"
;
int
   intPeriod               // Currunt Period Value
;
double
   dblPointDelta           // Current Value of a Tick Point
,  dblPipDelta             // Current Value of a Pip Point
,  dblPipMultiplier        // Pip/Point Adjustment Multiplier
,  dblMarketLotMin         // Market Minimum Lot Size
,  dblMarketLotMax         // Market Maximum Lot Size
,  dblMarketLotStep        // Market Lot Step
,  dblMarketStopLevel      // Market Minimum Stop Size (Delta)
,  dblMarketStopPips       // Market Minimum Stop Size (Pips)
;
void voidGlobalContextPrepare()
{
   nameSymbol  = Symbol();
   namePeriod  = strPeriodName( Period() );
   
   if (namePrefix != "") return;
   
   namePrefix  = _NAME_ + "." + nameSymbol + "." + namePeriod;
   
   // Define Values of a Pip and a Point
   dblPointDelta     = dblPointSize( nameSymbol );
   dblPipDelta       = dblPipSize( nameSymbol );
   dblPipMultiplier  = dblPipDelta / dblPointDelta;

   //---- Get Market Information
   dblMarketLotMin      = MarketInfo( nameSymbol, MODE_MINLOT );
   dblMarketLotMax      = MarketInfo( nameSymbol, MODE_MAXLOT );
   dblMarketLotStep     = MarketInfo( nameSymbol, MODE_LOTSTEP );
   dblMarketStopLevel   = NormalizeDouble( dblStopLevel( nameSymbol ), Digits );
   dblMarketStopPips    = dblMarketStopLevel / dblPipDelta;
   
   // Set Interval for Updateing Global Parameters
   if( !( IsTesting() || IsOptimization() || IsVisualMode() ) )
   {
      if( !EventSetTimer( intParamUpdateMins * 60 ) )
         voidPrint( "Error: Unable to Set Time for Global Parameter Update Interval!" );
   }
}


//+------------------------------------------------------------------------------------+
// Daily Breakout: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalDayBreakInitialise()
{
   //---- Only if in Live Trading
   if( !( IsTesting() || IsOptimization() || IsVisualMode() ) )
   {
      intExpirePendingDays = (int) dblGlobalVariableRead( nameExpirePendingDays,
                              intExpirePendingDays, namePrefix );
      intExpireOpenHours   = (int) dblGlobalVariableRead( nameExpireOpenHours,
                              intExpireOpenHours, namePrefix );
      intOpenHour          = (int) dblGlobalVariableRead( nameOpenHour,
                              intOpenHour, namePrefix );
      dblHighOffsetPips    = dblGlobalVariableRead( nameHighOffsetPips,
                              dblHighOffsetPips, namePrefix );
      dblLowOffsetPips     = dblGlobalVariableRead( nameLowOffsetPips,
                              dblLowOffsetPips, namePrefix );
   }
   
   //---- Adjust and Correct Parameter Values
   intExpirePendingDays = fabs( intExpirePendingDays );
   intExpireOpenHours   = fabs( intExpireOpenHours );

   if( intExpirePendingDays   < 1   )  intExpirePendingDays = 1;
   if( intExpireOpenHours     < 1   )  intExpireOpenHours   = 1;
   if( intOpenHour            < 0   )  intOpenHour          = -1;
   if( intOpenHour            > 23  )  intOpenHour          = 23;
   
   intExpireOpenSeconds = intExpireOpenHours * 60 * 60;
   
   //---- Define Price Delta Variables for PIP Values
   dblHighOffsetDelta   = NormalizeDouble( dblHighOffsetPips
                        * dblPipDelta, Digits);
   dblLowOffsetDelta    = NormalizeDouble( dblLowOffsetPips
                        * dblPipDelta, Digits);
}


//+------------------------------------------------------------------------------------+
// Spread Management: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalSpreadInitialise()
{
   //---- Only if in Live Trading
   if ( !( IsTesting() || IsOptimization() || IsVisualMode() ) )
   {
      dblSpreadMaximumPips    = dblGlobalVariableRead( nameSpreadMaximumPips,
                                 dblSpreadMaximumPips, namePrefix );
   }
   
   //---- Adjust and Correct Parameter Values                  
   dblSpreadMaximumPips    = fabs( dblSpreadMaximumPips );
   
   //---- Define Price Delta Variables for PIP Values
   dblSpreadMaximumDelta   = NormalizeDouble( dblSpreadMaximumPips
                              * dblPipDelta, Digits);
}


//+------------------------------------------------------------------------------------+
// Stops Management: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalStopsInitialise()
{
   //---- Only if in Live Trading
   if ( !( IsTesting() || IsOptimization() || IsVisualMode() ) )
   {
      dblStopMinimumPips      = dblGlobalVariableRead( nameStopMinimumPips,
                                 dblStopMinimumPips, namePrefix );
      dblSoftStopLossPips     = dblGlobalVariableRead( nameSoftStopLossPips,
                                 dblSoftStopLossPips, namePrefix );
      dblSoftTakeProfitPips   = dblGlobalVariableRead( nameSoftTakeProfitPips,
                                 dblSoftTakeProfitPips, namePrefix );
      dblSoftTrailStopPips    = dblGlobalVariableRead( nameSoftTrailStopPips,
                                 dblSoftTrailStopPips, namePrefix );
      dblSoftTrailStepPips    = dblGlobalVariableRead( nameSoftTrailStepPips,
                                 dblSoftTrailStepPips, namePrefix );
   }
   
   //---- Adjust and Correct Parameter Values                  
   dblStopMinimumPips      = fabs( dblStopMinimumPips );
   dblSoftStopLossPips     = fabs( dblSoftStopLossPips );
   dblSoftTakeProfitPips   = fabs( dblSoftTakeProfitPips );
   dblSoftTrailStopPips    = fabs( dblSoftTrailStopPips );
   dblSoftTrailStepPips    = fabs( dblSoftTrailStepPips );
   
   if ( dblStopMinimumPips    < dblMarketStopPips )
      dblStopMinimumPips      = dblMarketStopPips;
      
   if ( dblSoftStopLossPips   < dblStopMinimumPips )
      dblSoftStopLossPips     = dblStopMinimumPips;
               
   if ( dblSoftTakeProfitPips < dblStopMinimumPips )
      dblSoftTakeProfitPips   = dblStopMinimumPips;
      
   if ( dblSoftTrailStopPips  < dblStopMinimumPips )
      dblSoftTrailStopPips    = dblStopMinimumPips;

   if ( dblSoftTrailStepPips  < dblPipDelta )
      dblSoftTrailStepPips    = dblPipDelta;
      
   //---- Define Price Delta Variables for PIP Values
   dblStopMinimumDelta     = NormalizeDouble( dblStopMinimumPips
                           * dblPipDelta, Digits);
   dblSoftStopLossDelta    = NormalizeDouble( dblSoftStopLossPips
                           * dblPipDelta, Digits);
   dblSoftTakeProfitDelta  = NormalizeDouble( dblSoftTakeProfitPips
                           * dblPipDelta, Digits);
   dblSoftTrailStopDelta   = NormalizeDouble( dblSoftTrailStopPips
                           * dblPipDelta, Digits);
   dblSoftTrailStepDelta   = NormalizeDouble( dblSoftTrailStepPips
                           * dblPipDelta, Digits);
}


//+------------------------------------------------------------------------------------+
// Lots Management: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalLotsInitialise()
{
   //---- Only if in Live Trading
   if ( !( IsTesting() || IsOptimization() || IsVisualMode() ) )
   {
      dblLotsMinimum = dblGlobalVariableRead( nameLotsMinimum,
                     dblLotsMinimum, namePrefix );
      dblLotsMaximum = dblGlobalVariableRead( nameLotsMaximum,
                     dblLotsMaximum, namePrefix );
      dblLotsStep    = dblGlobalVariableRead( nameLotsStep,
                     dblLotsStep, namePrefix );
   }
   
   //---- Adjust and Correct Parameter Values                  
   dblLotsMinimum = MathFloor( fabs( dblLotsMinimum )
               * dblMarketLotStep ) / dblMarketLotStep;
   dblLotsMaximum = MathFloor( fabs( dblLotsMaximum )
               * dblMarketLotStep ) / dblMarketLotStep;
   dblLotsStep    = MathFloor( fabs( dblLotsStep )
               * dblMarketLotStep ) / dblMarketLotStep;
               
   if ( dblLotsMinimum  < dblMarketLotMin )  dblLotsMinimum = dblMarketLotMin;
   if ( dblLotsMinimum  > dblMarketLotMax )  dblLotsMinimum = dblMarketLotMax;
   if ( dblLotsMaximum  < dblMarketLotMin )  dblLotsMaximum = dblMarketLotMin;
   if ( dblLotsMaximum  > dblMarketLotMax )  dblLotsMaximum = dblMarketLotMax;
   if ( dblLotsStep     > dblMarketLotMax )  dblLotsStep    = dblMarketLotMax;
   if ( dblLotsStep     < dblMarketLotStep ) dblLotsStep    = dblMarketLotStep;
}


//+------------------------------------------------------------------------------------+
// Slippage Management: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalSlippageInitialise()
{
   //---- Only if in Live Trading
   if ( !( IsTesting() || IsOptimization() || IsVisualMode() ) )
   {
      dblSlippageEnterPips = dblGlobalVariableRead( nameSlippageEnterPips,
                           dblSlippageEnterPips, namePrefix );
      dblSlippageExitPips     = dblGlobalVariableRead( nameSlippageExitPips,
                           dblSlippageExitPips, namePrefix );
   }
   
   //---- Adjust and Correct Parameter Values
   dblSlippageEnterPips    = fabs( dblSlippageEnterPips );
   dblSlippageExitPips     = fabs( dblSlippageExitPips );
   
   //---- Define Price Delta Variables for PIP Values
   intSlippageEnterPoints  = (int) round( dblSlippageEnterPips * dblPipMultiplier );
   intSlippageExitPoints   = (int) round( dblSlippageExitPips * dblPipMultiplier );
}


//+------------------------------------------------------------------------------------+
// Money & Risk Management: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalMoneyRiskInitialise()
{
   //---- Only if in Live Trading
   if ( !( IsTesting() || IsOptimization() || IsVisualMode() ) )
   {
      dblRiskMaximum    = dblGlobalVariableRead( nameRiskMaximum,
                        dblRiskMaximum, namePrefix );
      dblMarginMaximum  = dblGlobalVariableRead( nameMarginMaximum,
                        dblMarginMaximum, namePrefix );
      dblBalanceMinimum = dblGlobalVariableRead( nameBalanceMinimum,
                        dblBalanceMinimum, namePrefix );
   }
}


//+------------------------------------------------------------------------------------+
// Magic Numbers: Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalMagicInitialise()
{
   //---- Only if in Live Trading
   if ( !( IsTesting() || IsOptimization() || IsVisualMode() ) )
   {
      intMagicMajor  = (int) dblGlobalVariableRead( nameMagicMajor,
                     intMagicMajor, namePrefix );
      intMagicMinor  = (int) dblGlobalVariableRead( nameMagicMinor,
                     intMagicMinor, namePrefix );
   }
   
   //---- Prepare Magic Numbers for current chart
   intMagicMask =
      (
         (  ( intMagicMajor * 16 + intMagicMajor ) * 16
            + intPeriodIndex( intPeriod )
         ) * 4096
      );
   intMagicFirst     = intMagicMask;
   intMagicLast      = intMagicMask + 4095;
   intMagicPrimary   = intMagicFirst;
}


//+------------------------------------------------------------------------------------+
// Initialise Global Context Variables (Client Terminal)
//+------------------------------------------------------------------------------------+
void voidGlobalContextInitialise()
{
   voidGlobalDayBreakInitialise();  // Initialise DayBreak Settings
   voidGlobalSpreadInitialise();    // Initialise Spread Management Settings
   voidGlobalStopsInitialise();     // Initialise Stops Management Settings
   voidGlobalLotsInitialise();      // Initialise Lots Management Settings
   voidGlobalSlippageInitialise();  // Initialise Slippage Management Settings
   voidGlobalMoneyRiskInitialise(); // Initialise Money & Risk Management Settings
   voidGlobalMagicInitialise();     // Initialise Magic Numbers Settings
   
   //---- Only if in Live Trading
   if ( !( IsTesting() || IsOptimization() || IsVisualMode() ) )
   {
      //---- Initialise & Retrieve Specific Variables
      enumTradingState
         = (ENUM_TRADING_STATE) dblGlobalVariableRead( nameTradingState, enumTradingState, namePrefix );

      //---- Initialise & Retrieve Overall Variables
      enumTradingStateAll
         = (ENUM_TRADING_STATE) dblGlobalVariableRead( nameTradingStateAll, enumTradingStateAll, "" );
      
      // Initialise the Comment Display
      voidCommentInitialise();
      
      GlobalVariablesFlush();
   }
}


//+------------------------------------------------------------------------------------+
// Display and Print Special Parameters
//+------------------------------------------------------------------------------------+
void voidCommentInitialise()
{
   if ( !( IsTesting() || IsOptimization() || IsVisualMode() ) )
   {
      string strComment = StringConcatenate( _NAME_,
         "; S: ", DoubleToString( ( Ask - Bid ) / dblPipDelta, 1 ),
         "; TP: ", DoubleToString( dblSoftTakeProfitPips, 1 ),
         "; SL: ", DoubleToString( dblSoftStopLossPips, 1 ),
         "; TS: ", DoubleToString( dblSoftTrailStopPips, 1 ),
         " (", DoubleToString( dblSoftTrailStepPips, 1 ), ")" );
      
      if( intDayHighIndex >= 0 )
         strComment = StringConcatenate( strComment,
         "; H: ", DoubleToString( dblDailyHighs[ intDayHighIndex ], Digits ) );
      
      if( intDayLowIndex >= 0 )
         strComment = StringConcatenate( strComment,
         "; L: ", DoubleToString( dblDailyLows[ intDayLowIndex ], Digits ) );
   
      //voidCommentPrint( strComment );
      voidCommentPrintString( strComment, true, false );
   }
}


//+------------------------------------------------------------------------------------+
// Startup Initialization function
//+------------------------------------------------------------------------------------+
bool
   boolStartInit = true // Procedural Initialisation Flags
;
void voidStartupInit()
{
   //---- Prepare Nameing and other Constants for Context Variables (Client Terminal)
   voidGlobalContextPrepare();      // Prepare prefix, names and other constants
   
   //---- Create and Define Global Context Variables (Client Terminal)
   voidGlobalContextInitialise();   // Initialize & retrieve values
   
   // Show or Hide Indicators depending on Environment
   if ( !( IsTesting() || IsOptimization() ) || IsVisualMode() )
      HideTestIndicators( false );
   else
      HideTestIndicators( true );
}


//+------------------------------------------------------------------------------------+
// Verify Current Daily High & Low
//+------------------------------------------------------------------------------------+
void voidCheckDayHighLow( bool boolBarNew, int intBarShift )
{
   static bool
      boolInitDay       = true   // Initialise Daily Data
   ,  boolInitDayHigh   = true   // Initialise Daily High Arrays
   ,  boolInitDayLow    = true   // Initialise Daily Low Arrays
   ;
   static int
      intHourStamp      = EMPTY  // Hour Stamp for New Hour detection
   ;

   bool boolExceptionFlag;       // Exception Flag
   
   // Check for New Bar
   if( boolBarNew || boolInitDay )
   {
      datetime dtTimeBar   = Time[ 0 ];               // Get Current Bar DateTime
      int      intHourBar  = TimeHour( dtTimeBar );   // Get Current Bar Hour
      
      if( boolInitDay || ( Period() >= PERIOD_D1 ) )
         boolExceptionFlag = true;
      else
         boolExceptionFlag = false;
      
      // Check for New Hour
      if( ( intHourStamp != intHourBar ) || boolExceptionFlag )
      {
         intHourStamp = intHourBar; // Update Hour Stamp
         
         int intHourSession = intOpenHour;
         if( intHourSession < 0 )
            intHourSession = TimeHour( iTime( NULL, PERIOD_W1, 0 ) );
         
         // Check for Session Openinig Hour
         if( ( intHourBar == intHourSession )  || boolExceptionFlag )
         {
            int intBarOffset = 1;
            
            if( boolExceptionFlag )
            {
               MqlDateTime mqldtBar;
               TimeToStruct( dtTimeBar, mqldtBar );
               
               mqldtBar.hour  = intHourSession;
               mqldtBar.min   = 0;
               mqldtBar.sec   = 0;
               
               datetime dtBarOpen = StructToTime( mqldtBar );
               
               if( dtBarOpen > dtTimeBar )
                  dtTimeBar -= 24*60*60;
               
               intBarOffset = iBarShift( NULL, PERIOD_H1, dtBarOpen ) + 1;
            }
            
            if( boolInitDay )
            {
               boolInitDayHigh   = true;
               boolInitDayLow    = true;
            }
         
            // Reset Array Size based on Expire Pending Days
            intDayArraySize = intExpirePendingDays;

            if( boolInitDayHigh || ( ArraySize( dblDailyHighs ) < intDayArraySize ) )
               ArrayResize( dblDailyHighs, intDayArraySize );
               
            if( boolInitDayLow || ( ArraySize( dblDailyLows ) < intDayArraySize ) )
               ArrayResize( dblDailyLows, intDayArraySize );

            double
               dblHighLimit   = Bid + dblStopMinimumDelta + dblHighOffsetDelta
            ,  dblLowLimit    = Bid - dblStopMinimumDelta - dblLowOffsetDelta
            ;
            
            if( boolInitDayHigh || ( intDayHighIndex >= ( intDayArraySize - 1 ) ) )
            {
               // Update Daily Highs
               intDayHighIndex = -1;
               double dblDayHigh;
               for( int i = 0, j = 0; i < intDayArraySize; i++, j += 24 )
               {
                  dblDayHigh = iHigh( NULL, PERIOD_H1,
                     iHighest( NULL, PERIOD_H1, MODE_HIGH, 24, intBarOffset + j ) );
                     
                  if( dblDayHigh > 0 )
                  {
                     if( intDayHighIndex >= 0 )
                     {
                        if( dblDayHigh <= dblDailyHighs[ intDayHighIndex ] )
                           continue;
                     }
   
                     intDayHighIndex++;
                     dblDailyHighs[ intDayHighIndex ] = dblDayHigh;
                  }
               }
               boolInitDayHigh = true;
            }
            else
            {
               intDayHighIndex++;
               dblDailyHighs[ intDayHighIndex ] = iHigh( NULL, PERIOD_H1,
                  iHighest( NULL, PERIOD_H1, MODE_HIGH, 24, intBarOffset ) );
                  
               if( dblDailyHighs[ intDayHighIndex ] <= dblHighLimit )
                  intDayHighIndex--;
            }
            
            if( boolInitDayLow || ( intDayLowIndex >= ( intDayArraySize - 1 ) ) )
            {
               // Update Daily Lows
               intDayLowIndex = -1;
               double dblDayLow;
               for( int i = 0, j = 0; i < intDayArraySize; i++, j += 24 )
               {
                  dblDayLow = iLow( NULL, PERIOD_H1,
                     iLowest( NULL, PERIOD_H1, MODE_LOW, 24, intBarOffset + j ) );
                     
                  if( dblDayLow > 0 )
                  {
                     if( intDayLowIndex >= 0 )
                     {
                        if( dblDayLow >= dblDailyLows[ intDayLowIndex ] )
                           continue;
                     }
   
                     intDayLowIndex++;
                     dblDailyLows[ intDayLowIndex ] = dblDayLow;
                  }
               }
               boolInitDayLow = true;
            }
            else
            {
               intDayLowIndex++;
               dblDailyLows[ intDayLowIndex ] = iLow( NULL, PERIOD_H1,
                  iLowest( NULL, PERIOD_H1, MODE_LOW, 24, intBarOffset ) );
                  
               if( dblDailyLows[ intDayLowIndex ] >= dblLowLimit )
                  intDayLowIndex--;
            }
            
            // Sort Arrays
            ArraySort( dblDailyHighs, intDayHighIndex + 1, 0, MODE_DESCEND );
            ArraySort( dblDailyLows, intDayLowIndex + 1, 0, MODE_ASCEND );
            
            if( boolInitDayHigh || boolInitDayLow )
            {
               // Set Indices into Array
               if( boolInitDayHigh )
               {
                  if( intDayHighIndex >= 0 )
                     while( dblDailyHighs[ intDayHighIndex ] <= dblHighLimit )
                     {
                        intDayHighIndex--;
                        if( intDayHighIndex < 0 ) break;
                     }
                  
                  boolInitDayHigh = false;
               }
               
               if( boolInitDayLow )
               {
                  if( intDayLowIndex >= 0 )
                     while( dblDailyLows[ intDayLowIndex ] >= dblLowLimit )
                     {
                        intDayLowIndex--;
                        if( intDayLowIndex < 0 ) break;
                     }
               
                  boolInitDayLow = false;
               }
            }
            
            if( boolInitDay )
            {
               boolInitDay       = false;
               boolInitDayHigh   = false;
               boolInitDayLow    = false;
            }
         }
      }
   }
}


//+------------------------------------------------------------------------------------+
// Verify Breakout Conditions set Signals accordingly
//+------------------------------------------------------------------------------------+
bool boolCheckBreakout()
{
   double dblBreakPrice;
   
   enumSignalDayBreak = _SIDE_OUT_;
   
   RefreshRates();
   
   if( intDayHighIndex >= 0 )
   {
      dblBreakPrice = Bid - dblHighOffsetDelta;
      
      while( dblBreakPrice > dblDailyHighs[ intDayHighIndex ] )
      {
         /*
         voidPrint( StringConcatenate(
            "[Break] Slip High:"
         ,  DoubleToString( ( Bid - dblDailyHighs[ intDayHighIndex ] ) / dblPipDelta, 1 )
         ) );
         */
         
         enumSignalDayBreak = _UP_LONG_;
         intDayHighIndex--;
         if( intDayHighIndex < 0 ) break;
      }
      
      /*
      if( enumSignalDayBreak == _UP_LONG_)
         voidPrint( StringConcatenate(
            "[Break] Remove High: ", intDayHighIndex + 1
         ,  " (", intDayArraySize, ")"
         ,  ", Bid: ", DoubleToString( Bid, Digits )
         ,  ", Ask: ", DoubleToString( Ask, Digits )
         ,  ", Offset: ", DoubleToString( dblHighOffsetDelta, Digits )
         ) );
      */
   }
   
   if( intDayLowIndex >= 0 )
   {
      dblBreakPrice = Bid + dblLowOffsetDelta;
      
      while( dblBreakPrice < dblDailyLows[ intDayLowIndex ] )
      {
         /*
         voidPrint( StringConcatenate(
            "[Break] Slip Low:"
         ,  DoubleToString( ( dblDailyLows[ intDayLowIndex ] - Bid ) / dblPipDelta, 1 )
         ) );
         */
         
         enumSignalDayBreak = _DOWN_SHORT_;
         intDayLowIndex--;
         if( intDayLowIndex < 0 ) break;
      }
      
      /*
      if( enumSignalDayBreak == _DOWN_SHORT_)
         voidPrint( StringConcatenate(
            "[Break] Remove Low: ", intDayLowIndex + 1
         ,  " (", intDayArraySize, ")"
         ,  ", Bid: ", DoubleToString( Bid, Digits )
         ,  ", Ask: ", DoubleToString( Ask, Digits )
         ,  ", Offset: ", DoubleToString( dblLowOffsetDelta, Digits )
         ) );
      */
   }

   return( enumSignalDayBreak != _SIDE_OUT_);
}


//+------------------------------------------------------------------------------------+
// Verify Current Spread and set Signal accordingly
//+------------------------------------------------------------------------------------+
bool boolCheckSpread()
{
   RefreshRates();

   dblSpreadDelta = dblSpreadPrice( Ask, Bid );
   dblSpreadPips  = dblSpreadDelta / dblPipDelta;

   // voidPrint( StringConcatenate( "Spread: ", DoubleToString( dblSpreadPips, 1 ) ) );

   boolSignalSpread = ( dblSpreadPips <= dblSpreadMaximumPips );
   
   return( boolSignalSpread );
}


//+------------------------------------------------------------------------------------+
// Function to Process Indicator Signals and BUY/SELL Signals
//+------------------------------------------------------------------------------------+
ENUM_TRADING_SIGNAL
   enumSignalEnter = _SIDE_OUT_
;
bool boolProcessSignals( bool boolBarNew, int intBarShift )
{
   // Reset Signals
   enumSignalEnter = _SIDE_OUT_;
   
   // Check & Update Daily Highs & Lows
   voidCheckDayHighLow( boolBarNew, intBarShift );
   voidCommentInitialise();
   
   if( boolCheckBreakout() )     // Check Breakout
   {
      if( boolCheckSpread() ) // Check Spread
         enumSignalEnter = enumSignalDayBreak;
   }
   
   return( enumSignalEnter != _SIDE_OUT_ );
}


//+------------------------------------------------------------------------------------+
// Process Bar function
//+------------------------------------------------------------------------------------+
void voidProcessBarTick( bool boolBarNew, int intBarShift )
{
   if (  ( enumTradingStateAll != _TRADING_NONE_ )
      && ( enumTradingState    != _TRADING_NONE_ ) )
   {
      if( boolProcessSignals( boolBarNew, intBarShift ) )
      {
         if( enumSignalEnter  != _SIDE_OUT_ )
         {
            intPlaceOrder( enumSignalEnter );
         }
      }
   }
   
   // Process Open Orders once per tick
   voidOrdersProcessOnTick();
}


//+------------------------------------------------------------------------------------+
// Order Management Functions
//+------------------------------------------------------------------------------------+
#define  _ORDERS_BUFSIZE_  1024
int
   intOrdersCount = 0      // Size of Order Arrays
,  intOrdersLast
;


//+------------------------------------------------------------------------------------+
// Function to Place Order
//+------------------------------------------------------------------------------------+
int intPlaceOrder( ENUM_TRADING_SIGNAL enumSignal )
{
   double   dblLots, dblPrice, dblStopLoss, dblTakeProfit;
   int      intType, intColour;
   string   strComment;
   datetime dtExpiration = TimeCurrent() + intExpireOpenSeconds - 1;
   
   RefreshRates();
   
   switch( enumSignal )
   {
      case _UP_LONG_:
         dblPrice             = Ask;
         intType              = OP_BUY;
         intColour            = MediumSeaGreen;
         strComment           = StringConcatenate( namePrefix, ".Long." );
         break;
   
      case _DOWN_SHORT_:
         dblPrice             = Bid;
         intType              = OP_SELL;
         intColour            = MediumVioletRed;
         strComment           = StringConcatenate( namePrefix, ".Short." );
         break;
      
      default:
         return( EMPTY );
   }
   
   dblLots        = dblLotsRisk(
                     nameSymbol, dblSoftStopLossPips,
                     dblRiskMaximum, dblMarginMaximum,
                     dblLotsMinimum, dblLotsMaximum, dblLotsStep );
                     
   int
      intTicket            = EMPTY
   ,  intOrderRetryCount   = intRetryCount
   ;
   
   if( IsTesting() || IsOptimization() ) intOrderRetryCount = 0;
            
   do
   {
      intTicket = OrderSend(
            nameSymbol, intType, dblLots, dblPrice,
            intSlippageEnterPoints, 0, 0, strComment,
            intMagicPrimary, dtExpiration, intColour );
            
      if( intTicket != EMPTY ) break;  
      intOrderRetryCount--;
      if( intOrderRetryCount > 0 )
         voidSleepRandom( dblSleepTimeMean, dblSleepTimeMax );
   }
   while( intOrderRetryCount > 0 );
   
   if( intTicket != EMPTY )
   {
      if( IsTesting() || IsOptimization() )
         intOrderRetryCount = 0;
      else
         intOrderRetryCount = intRetryCount;
   
      do
      {
         if( OrderSelect( intTicket, SELECT_BY_TICKET, MODE_TRADES ) )
         {
            dblPrice       = OrderOpenPrice();
            dblStopLoss    = NormalizeDouble( dblPrice
                              - dblSoftStopLossDelta * enumSignal, Digits );
            dblTakeProfit  = NormalizeDouble( dblPrice
                              + dblSoftTakeProfitDelta * enumSignal, Digits );
            
            if( IsTesting() || IsOptimization() )
               intOrderRetryCount = 0;
            else
               intOrderRetryCount = intRetryCount;
            
            do
            {
               if( OrderModify( intTicket, dblPrice,
                     dblStopLoss, dblTakeProfit, dtExpiration, intColour ) )
                  return( intTicket );
               intOrderRetryCount--;
               if( intOrderRetryCount > 0 )
                  voidSleepRandom( dblSleepTimeMean, dblSleepTimeMax );
            }
            while( intOrderRetryCount > 0 );
         }
         intOrderRetryCount--;
         if( intOrderRetryCount > 0 )
            voidSleepRandom( dblSleepTimeMean, dblSleepTimeMax );
      }
      while( intOrderRetryCount > 0 );
      
      voidPrint( StringConcatenate(
            "intPlaceOrder: OrderModify Error: ", ErrorDescription( GetLastError() )
            ) );
   }
   else
   {
      voidPrint( StringConcatenate(
            "intPlaceOrder: OrderSend Error: ", ErrorDescription( GetLastError() )
            ) );
   }
            
   return( intTicket );
}


//+------------------------------------------------------------------------------------+
// Function to Verify and Close Order for particular Ticket, Magic Number and Type
//+------------------------------------------------------------------------------------+
bool boolVerifyCloseOrder( int intTicket, int intMagic, int intType )
{
   if ( OrderSelect( intTicket, SELECT_BY_TICKET, MODE_TRADES ) )
   {
      if (     ( OrderSymbol()      == nameSymbol  )
            && ( OrderMagicNumber() == intMagic    )
            && ( OrderType()        == intType     )  )
      {
         int intOrderRetryCount = intRetryCount;
         if ( IsTesting() || IsOptimization() ) intOrderRetryCount = 0;
         
         if ( ( intType == OP_BUY ) || ( intType == OP_SELL ) )
         {
      	   do
            {
               if( OrderClose( intTicket, OrderLots(), OrderClosePrice(),
                     intSlippageExitPoints, DeepPink ) )
                  return( true );
               intOrderRetryCount--;
               if( intOrderRetryCount > 0 )
                  voidSleepRandom( dblSleepTimeMean, dblSleepTimeMax );
            }
            while( intOrderRetryCount > 0 );
         }
         else
         {
            do
            {
               if( OrderDelete( intTicket ) ) return( true );
               intOrderRetryCount--;
               if( intOrderRetryCount > 0 )
                  voidSleepRandom( dblSleepTimeMean, dblSleepTimeMax );
            }
            while( intOrderRetryCount > 0 );
         }
            
         voidPrint( StringConcatenate(
            "boolVerifyCloseOrder: Order Close/Delete Error: ",
            ErrorDescription( GetLastError() )
            ) );
            
         return( false );
      }
      
      voidPrint( "boolVerifyCloseOrder: Invalid/Incorrect Symbol, Type or Magic number!" );
   }

   voidPrint( StringConcatenate(
      "boolVerifyCloseOrder: OrderSelect Error: ", ErrorDescription( GetLastError() )
      ) );
	            
   return( false );
}


//+------------------------------------------------------------------------------------+
// Function to Process Open Orders on Every Tick one at a time
//+------------------------------------------------------------------------------------+
void voidOrdersProcessOnTick()
{
   // Static Variables
   static int intIndex = 0;
   
   // Dynamic Variables
   bool  boolUpdate, boolClose;
   int   intMagic, intType, intDirection;
   double
      dblOpen, dblClose, dblPriceDelta, dblTrailDelta, dblTrailPrice, 
      dblStopDelta, dblTakeProfit, dblStopLoss;
   
   intIndex--; // Next Order
   
   if( intIndex < 0 )
   {
      intIndex = OrdersTotal() - 1; // Restart from End
      if( intIndex < 0 ) return;
   }
      
   if( OrderSelect( intIndex, SELECT_BY_POS, MODE_TRADES ) )
   {
      // Check if Open
      if( OrderCloseTime() != 0 ) return;

      // Check Expiration Date/Time
      datetime dtExpiration = OrderExpiration();
      if( dtExpiration != NULL )
      {
         // Honour expiration time
         if( ( dtExpiration < TimeCurrent() ) || ( dtExpiration < Time[ 0 ] ) )
         {
            boolVerifyCloseOrder( OrderTicket(), OrderMagicNumber(), OrderType() );
            return;
         }
      }

      // Check if Right Symbol
      if( OrderSymbol() != nameSymbol )   return;
         
      // Check Magic Number
      intMagic = OrderMagicNumber();
      if(   ( intMagic  < intMagicFirst   )  ||
            ( intMagic  > intMagicLast    )     )  return;
         
      // Check Order Type
      intType  = OrderType();
      switch( intType )
      {
         case OP_BUY:
         case OP_BUYSTOP:
         case OP_BUYLIMIT:
            intDirection = _UP_LONG_;
            break;
      
         case OP_SELL:
         case OP_SELLSTOP:
         case OP_SELLLIMIT:
            intDirection = _DOWN_SHORT_;
            break;
            
         default:
            return;
      }
      
      dblOpen        = OrderOpenPrice();
      dblStopLoss    = OrderStopLoss();
      dblTakeProfit  = OrderTakeProfit();
      boolUpdate     = false;
      boolClose      = false;

      dblStopDelta   = dblSoftStopLossDelta - dblSoftTrailStopDelta;

      RefreshRates();
      
      dblClose       = OrderClosePrice();
      dblPriceDelta  = ( dblClose - dblOpen ) * intDirection;
      if( dblStopDelta > 0 ) dblPriceDelta -= dblStopDelta;
      
      dblTrailDelta  = floor( dblPriceDelta / dblSoftTrailStepDelta ) 
                     * dblSoftTrailStepDelta - dblSoftTrailStopDelta;
      dblTrailPrice  = dblOpen + dblTrailDelta * intDirection;

      if( dblStopLoss == 0 )
      {
         switch( intDirection )
         {
            case _UP_LONG_:
               dblStopLoss = fmin(
                  dblOpen - dblSoftStopLossDelta,
                  dblClose - dblStopMinimumDelta );
               break;
            
            case _DOWN_SHORT_:
               dblStopLoss = fmax(
                  dblOpen + dblSoftStopLossDelta,
                  dblClose + dblStopMinimumDelta );
               break;
         }
         boolUpdate = true;
      }
      
      if( dblTakeProfit == 0 )
      {
         switch( intDirection )
         {
            case _UP_LONG_:
               dblTakeProfit = fmax(
                  dblOpen + dblSoftTakeProfitDelta,
                  dblClose + dblStopMinimumDelta );
               break;
            
            case _DOWN_SHORT_:
               dblTakeProfit = fmin(
                  dblOpen - dblSoftTakeProfitDelta,
                  dblClose - dblStopMinimumDelta );
               break;
         }
         boolUpdate = true;
      }
      
      switch( intDirection )
      {
         case _UP_LONG_:
            if( dblTrailPrice > dblStopLoss )
            {
               if( dblTrailPrice >= dblOpen )
               {
                  if( dblClose > dblTrailPrice )
                  {
                     dblStopLoss = fmin( dblTrailPrice, dblClose - dblStopMinimumDelta );
                     boolUpdate  = true;
                  }
                  else
                     boolClose = true;
               }
            }
            break;
      
         case _DOWN_SHORT_:
            if( dblTrailPrice < dblStopLoss )
            {
               if( dblTrailPrice <= dblOpen )
               {
                  if( dblClose < dblTrailPrice )
                  {
                     dblStopLoss = fmax( dblTrailPrice, dblClose + dblStopMinimumDelta );
                     boolUpdate  = true;
                  }
                  else
                     boolClose = true;
               }
            }
            break;
      }
      
      if( boolClose )
      {
         if( boolVerifyCloseOrder( OrderTicket(), intMagic, intType ) )
            boolUpdate  = false;
      }
      
      if ( boolUpdate )
      {
         dblStopLoss    = NormalizeDouble( dblStopLoss, Digits );
         dblTakeProfit  = NormalizeDouble( dblTakeProfit, Digits );
         
         if( !boolDblEqual( dblStopLoss, OrderStopLoss(), Digits     )  ||
            !boolDblEqual( dblTakeProfit, OrderTakeProfit(), Digits  )     )
         {
            int intOrderRetryCount = intRetryCount;
            if( IsTesting() || IsOptimization() ) intOrderRetryCount = 0;
            
            do
            {
               if( OrderModify( OrderTicket(), dblOpen,
                     dblStopLoss, dblTakeProfit, OrderExpiration(), Gold ) )
                  return;
               intOrderRetryCount--;
               if( intOrderRetryCount > 0 )
                  voidSleepRandom( dblSleepTimeMean, dblSleepTimeMax );
            }
            while( intOrderRetryCount > 0 );
            
            voidPrint( StringConcatenate(
               "voidOrdersProcessOnTick: OrderModify Error",
               "; Direction=", intDirection,
               "; SL=", DoubleToString( dblStopLoss, Digits ),
               "; TP=", DoubleToString( dblTakeProfit, Digits ),
               "; Ask=", DoubleToString( Ask, Digits ),
               "; Bid=", DoubleToString( Bid, Digits ),
               "; Error: ", ErrorDescription( GetLastError() )
               ) );
         }
      }
   }
}


//+------------------------------------------------------------------------------------+
// Process Statistics for Deinitialisation
//+------------------------------------------------------------------------------------+
double dblOnTesterValue;
void voidProcessStats()
{
   int intTradesTotal = (int) TesterStatistics( STAT_TRADES );
   
   if( intTradesTotal > 0 )
      dblOnTesterValue  = 100.0 * (double) TesterStatistics( STAT_PROFIT_TRADES )
                        / (double) intTradesTotal;
   else
      dblOnTesterValue  = 0;

   return;
}


//+------------------------------------------------------------------------------------+
// Tester Event Function
//+------------------------------------------------------------------------------------+

double OnTester()
{
   voidProcessStats();  // Process and Log Final Statistics
   return( dblOnTesterValue );
}


//+------------------------------------------------------------------------------------+
// Check minimum Trading Conditions
//+------------------------------------------------------------------------------------+
bool boolTradingConditionsValid()
{
   //---- If Testing, then ignore minimum requirements
   if ( IsTesting() || IsOptimization() || IsVisualMode() )
      return(true);

   //---- Check if Connected
   if ( !IsConnected() )
   {
      voidCommentPrint("NOT Trading (NOT connected)!");
      return(false);
   }

   //---- Check if Terminal Allows Trading
   /*if  ( !IsTradeAllowed() )
   {
      voidCommentPrint("NOT Trading (Disallowed by terminal)!");
      return(false);
   }*/
   
   //---- Check Global Overall Trading State 
   if ( enumTradingStateAll == _TRADING_NONE_ )
   {
      voidCommentPrint("NOT Trading (Overall trading state disabled)!");
      return(false);
   }
   
   //---- Check Global Current Trading State 
   if ( enumTradingState == _TRADING_NONE_ )
   {
      voidCommentPrint("NOT Trading (Current trading state disabled)!");
      return(false);
   }

   return(true);  // Otherwise, proceed with trading
}


//+-----------------------------------------------------------------------+
// Check minimum Trade Entering Conditions
//+-----------------------------------------------------------------------+
bool boolTradeEnterConditionsValid()
{
   //---- If Testing, then ignore minimum requirements
   if ( IsTesting() || IsOptimization() || IsVisualMode() )
      return(true);

   //---- Check if Connected
   if ( !IsConnected() )
   {
      voidCommentPrint("NOT Entering Trade (NOT connected)!");
      return(false);
   }

   //---- Check if Terminal Allows Trading
   /*if  ( !IsTradeAllowed() )
   {
      voidCommentPrint("NOT Entering Trade (Disallowed by terminal)!");
      return(false);
   }*/
   
   //---- Check Global Overall Trading State 
   if ( enumTradingStateAll != _TRADING_FULL_ )
   {
      voidCommentPrint("NOT Entering Trade (Overall trading state)!");
      return(false);
   }
   
   //---- Check Global Current Trading State 
   if ( enumTradingState != _TRADING_FULL_ )
   {
      voidCommentPrint("NOT Entering Trade (Current trading state)!");
      return(false);
   }

   //---- Check if Accound Balance is acceptable
   double dblValueAccount
      = fmin( fmin( AccountEquity(), AccountBalance() ), AccountFreeMargin() );
   if ( dblValueAccount < dblBalanceMinimum )
   {
      voidCommentPrint("NOT Entering Trade (Balance too low)!");
      return(false);
   }
   
   return(true);  // Otherwise, proceed with entering trade
}


//+------------------------------------------------------------------------------------+
// Initialisation Event Function
//+------------------------------------------------------------------------------------+
void OnInit()
{
   // Allow for remainder of initialisation to be done on the first tick
   boolStartInit = true;
   
   return;  // Initialization complete
}


//+------------------------------------------------------------------------------------+
// Tick Event Function
//+------------------------------------------------------------------------------------+
void OnTick()
{
   // Do Startup Initialisation
   if ( boolStartInit )
   {
      voidPrint( StringConcatenate(
            "Bars: ", Bars ) );
      voidStartupInit();
      boolStartInit = false;
   }
   
   //---- Check for a New Bar open
   static   datetime dateTimeStamp;    // TimeStamp for New Bar detection
            bool     boolBarNew;       // Consider not a New Bar Opening
            int      intBarShift = 1;  // Shift to Consider Previous Bar

   if (dateTimeStamp != Time[0]) // Check Bar Time if New Bar
   {
      dateTimeStamp  = Time[0];  // Time Stamp of Bar Opening
      boolBarNew     = true;     // Tick is first on New Bar
   }
   else
      boolBarNew     = false;
   
   //---- Check Minimum Requiments or Conditions for Trading
   if ( !boolTradingConditionsValid() )
      return;  // Do NOT trade, Conditions are not Valid
      
   // Only needed in Real Execution, not in Testing Mode
   if ( !( IsTesting() || IsOptimization() || IsVisualMode() ) )
   {
      if ( boolBarNew ) // Only once per bar
         voidGlobalContextInitialise();   // Initialize & retrieve values
   }
   
   //---- Process Details for New Bar and Every Tick positions
   voidProcessBarTick( boolBarNew, intBarShift );  // Process for new bar and tick
   
   //--- Processing Complete
   return;
}


//+------------------------------------------------------------------------------------+
// Timer Event Function
//+------------------------------------------------------------------------------------+
void OnTimer()
{
   voidGlobalContextInitialise();   // Initialize & retrieve values
}


//+------------------------------------------------------------------------------------+
// Deinitialisation Function
//+------------------------------------------------------------------------------------+
void OnDeinit( const int intReason )
{
   EventKillTimer(); // Killer the Timer Event
   return;  // Deinitialisation complete
}


//+------------------------------------------------------------------------------------+
//	Functions for Normalising Volume or Lot Sizing
//+------------------------------------------------------------------------------------+

	// Factor and Normalise the Lot 
	double dbLotsNormalize( double dblFactor, double dblSizeLots,
		double dblMinLots, double dblMaxLots, double dblStepLots )
	{
		if ( dblFactor == 0 ) return( 0 );
		double dblCalcLots
			= MathRound( dblFactor * dblSizeLots / dblStepLots ) * dblStepLots;
		if ( dblCalcLots > dblMaxLots )	dblCalcLots = dblMaxLots;
		if ( dblCalcLots < dblMinLots )	dblCalcLots = dblMinLots;
		return( dblCalcLots );
	}

	// Factor and Normalise (Ceiling) the Lot 
	double dbLotsNormalizeCeil( double dblFactor, double dblSizeLots,
		double dblMinLots, double dblMaxLots, double dblStepLots )
	{
		if ( dblFactor == 0 ) return( 0 );
		double dblCalcLots
			= MathCeil( dblFactor * dblSizeLots / dblStepLots ) * dblStepLots;
		if ( dblCalcLots > dblMaxLots )	dblCalcLots = dblMaxLots;
		if ( dblCalcLots < dblMinLots )	dblCalcLots = dblMinLots;
		return( dblCalcLots );
	}

	// Calculate Max Lot Size based on Maximum Risk
	double dblLotsRisk( string strSymbol, double dblStopLossPips,
		double dblRiskMaxPercent, double dblMarginMaxPercent,
		double dblMinLots, double dblMaxLots, double dblStepLots )
	{
		double
			dblValueAccount	= MathMin(
				MathMin( AccountEquity(), AccountBalance() ), AccountFreeMargin() )
		,	dblValueRisk	= dblValueAccount * dblRiskMaxPercent / 100.0
		,	dblValueMargin	= dblValueAccount * dblMarginMaxPercent / 100.0
		,	dblLossOrder	= dblStopLossPips * dblPipValue( strSymbol )
		,	dblMarginOrder	= MarketInfo( strSymbol, MODE_MARGINREQUIRED )
		,	dblCalcLotMin
				= MathMax( dblMinLots, MarketInfo( strSymbol, MODE_MINLOT ) )
		,	dblCalcLotMax
				= MathMin( dblMaxLots, MarketInfo( strSymbol, MODE_MAXLOT ) )
		,	dblModeLotStep	= MarketInfo( strSymbol, MODE_LOTSTEP )
		,	dblCalcLotStep	= MathCeil( dblStepLots / dblModeLotStep ) * dblModeLotStep
		,	dblCalcLotLoss
				= MathFloor( dblValueRisk / dblLossOrder / dblCalcLotStep ) * dblCalcLotStep
		,	dblCalcLotMargin
				= MathFloor( dblValueMargin / dblMarginOrder / dblCalcLotStep ) * dblCalcLotStep
		,	dblCalcLot = MathMin( dblCalcLotLoss, dblCalcLotMargin )
		;
		
		if ( dblCalcLot < dblCalcLotMin ) dblCalcLot = dblCalcLotMin;
		if ( dblCalcLot > dblCalcLotMax ) dblCalcLot = dblCalcLotMax;

		return ( dblCalcLot );
	}
	
	// Calculate Available Risk Value in PipLots
	double dbRiskPipLots( string strSymbol, double dblRiskMaxPercent )
	{
		return( MathMin( MathMin( AccountBalance(), AccountEquity() ),
					AccountFreeMargin() ) * dblRiskMaxPercent
					/ ( dblPipValue( strSymbol ) * 100.0 ) );
	}
	
	// Calculate Balance in PipLots
	double dbBalancePipLots( string strSymbol )
	{
		return( AccountBalance() / dblPipValue( strSymbol ) );
	}

	// Calculate Equity in PipLots
	double dbEquityPipLots( string strSymbol )
	{
		return( AccountEquity() / dblPipValue( strSymbol ) );
	}
	
	// Calculate Free Margin in PipLots
	double dbFreeMarginPipLots( string strSymbol )
	{
		return( AccountFreeMargin() / dblPipValue( strSymbol ) );
	}
	
	
//+------------------------------------------------------------------------------------+
//	Functions for Calculating Stop Level, Swaps and Required Margin
//+------------------------------------------------------------------------------------+

	// Function to Determine Current Broker Stop Level
	double dblStopLevel( string strSymbol )
	{	return( MarketInfo( strSymbol, MODE_STOPLEVEL )
			* dblPointSize( strSymbol ) );	}

	// Function to Determine Current Broker Margin Value Required
	double dblMarginRequired( string strSymbol )
	{	return( MarketInfo( strSymbol, MODE_MARGINREQUIRED ) );	}
	
	
//+------------------------------------------------------------------------------------+
//	Variables and Functions for Tick, Point and Pip Sizes and Values
//+------------------------------------------------------------------------------------+

	// Function to return Tick Size
	double dblTickSize( string strSymbol )
	{	return( MarketInfo( strSymbol, MODE_TICKSIZE ) );	}

	// Function to return Point Size
	double dblPointSize( string strSymbol )
	{	return( MarketInfo( strSymbol, MODE_POINT ) );	}

	// Function to return Pip Size
	double dblPipSize( string strSymbol )
	{
		double dblCalcPipPoint = dblPointSize( strSymbol );
		int intSymbolDigits = (int) MarketInfo( strSymbol, MODE_DIGITS );
		switch( intSymbolDigits )
		{
			case 3:
			case 5:
				dblCalcPipPoint *= 10;
				break;
		}
		return( dblCalcPipPoint );
	}
	
	// Function to return Pip Digits
	int dblPipDigits( string strSymbol )
	{
		int intCalcPipDigits = 0;
		int intSymbolDigits = (int) MarketInfo( strSymbol, MODE_DIGITS );
		switch( intSymbolDigits )
		{
			case 3:
			case 5:
				intCalcPipDigits = 1;
				break;
		}
		return( intCalcPipDigits );
	}

	// Function to Determine Tick Point Value in Account Currency
	double dblTickValue( string strSymbol )
	{
		if ( IsTesting() )
		{
			if( StringSubstr( strSymbol, 3 ) == AccountCurrency() )
			{
			   return( 
			      ( dblPipSize( strSymbol ) / 0.0001 )
			      / MarketInfo( Symbol(), MODE_BID )  );
			}
		}

		return( MarketInfo( strSymbol, MODE_TICKVALUE ) );
	}

	// Function to Determine Point Value in Account Currency
	double dblPointValue( string strSymbol )
	{
		return(	dblTickValue( strSymbol ) * dblPointSize( strSymbol ) 
				/ dblTickSize( strSymbol ) );
	}

	// Function to Determine Pip Point Value in Account Currency
	double dblPipValue( string strSymbol )
	{
		double dblCalcPipValue = dblPointValue( strSymbol );
		int intSymbolDigits = (int) MarketInfo( strSymbol, MODE_DIGITS );
		switch ( intSymbolDigits )
		{
			case 3:
			case 5:
				dblCalcPipValue *= 10;
				break;
		}
		
		return( dblCalcPipValue );
	}


//+------------------------------------------------------------------------------------+
//	Variables and Functions for Handling available Time Frame Periods
//+------------------------------------------------------------------------------------+

	// Define Period Values as per defined Contants
	int
		intPeriodValues[] = {
			PERIOD_M1,  PERIOD_M5,  PERIOD_M15, PERIOD_M30,
			PERIOD_H1,  PERIOD_H4,  PERIOD_D1,  PERIOD_W1, PERIOD_MN1
		};
                     
	// Define Period Names/Codes for above Values
	string
		strPeriodNames[] = {
			"M1", "M5", "M15", "M30", "H1", "H4", "D1", "W1", "MN1"
		};
	
	// Function to Return Period Index based on Period Value
	int intPeriodIndex( int intPeriodValue )
	{
		int intPos = ArrayBsearch( intPeriodValues, intPeriodValue );
		if ( intPeriodValues[intPos] != intPeriodValue )
			return( EMPTY );
		return( intPos );
	}

	// Function to Return Period Name based on Period Value
	string strPeriodName( int intPeriodValue )
	{
		int intPos = ArrayBsearch( intPeriodValues, intPeriodValue );
		if ( intPeriodValues[intPos] != intPeriodValue )
			return("Unknown?");
		return( strPeriodNames[intPos] );
	}


//+------------------------------------------------------------------------------------+
//	Variables and Functions for Handling Global Context Variables
//+------------------------------------------------------------------------------------+

	// Variable to Identify when Global Context has changed ( Dirty Flag )
	bool boolGlobalDirtyFlag = false;

	// Function to Retrieve Global Context Dirty Flag ( and Reset it if required )
	bool boolGlobalDirty( bool boolReset = false )
	{
		if ( !boolGlobalDirtyFlag )
			return( false );
			
		if ( boolReset )
			boolGlobalDirtyFlag = false;
			
		return( true );
	}

	// Function to Retrieve Global Context Variable, set it if non-existing
	double dblGlobalVariableRead( string strName,
		double dblValue = 0, string strPrefix = "" )
	{
		string strID = StringConcatenate( strPrefix, ".", strName );
		if ( !GlobalVariableCheck( strID ) )
			GlobalVariableSet( strID, dblValue );
			
		double dblValueNew = GlobalVariableGet( strID );
		
		if ( ( dblValueNew - dblValue ) != 0.0 )
			boolGlobalDirtyFlag = true;
		
		return( dblValueNew );
	}
	

//+------------------------------------------------------------------------------------+
//	Functions for Handling Logs and Comments
//+------------------------------------------------------------------------------------+

   // Function to Print to Log and/or Comment to Chart a String
	void voidCommentPrintString(
			string strText, bool boolComment = true, bool boolPrint = true )
	{
		if( IsOptimization() ) return;
		if( IsTesting() &&  !IsVisualMode() ) boolComment = false;
		
		if ( boolComment )	Comment( strText );
		if ( boolPrint )	   Print( strText );
	}

	// Function to Print to Log a String
	void voidPrint( string strText )
	{
		voidCommentPrintString( strText, false, true );
	}
	
	// Function to Print to Log and Comment to Chart a String
	void voidCommentPrint( string strText )
	{
		voidCommentPrintString( strText, true, true );
	}
	
	
//+------------------------------------------------------------------------------------+
//	Extra Support Variables and Functions
//+------------------------------------------------------------------------------------+

   // Functions to Calculate Price Spread
	double dblSpreadPrice( double dblAskPrice, double dblBidPrice )
	{	return( MathAbs( dblAskPrice - dblBidPrice ) );	}
	
	// Compare if two double values are Equal
	bool boolDblEqual( double dblValue1, double dblValue2, int intDigits = -1 )
	{
		if ( intDigits < 0 )
			intDigits = (int) MarketInfo( Symbol(), MODE_DIGITS );	// "Digits"
			
		return( NormalizeDouble( dblValue1 - dblValue2, intDigits )
		          == NormalizeDouble( 0, Digits ) );
	}
	
	// Function for Random Sleep Time
   void voidSleepRandom( double dblTimeMean, double dblTimeMax )
   {
	   if ( IsTesting() || IsOptimization() ) return;
	   
   	double dblTenths = ceil( dblTimeMean * 10 );
   	if ( dblTenths <= 0 ) return;

   	int intTenthsMax = (int) round( dblTimeMax * 10 );
   	double
   	   dblP     = 1.0 - 1.0 / (double) intTenthsMax
   	,  dblPrnd  = dblP * 32768
   	;

	   Sleep(10); // Always sleep at least one-hundredth second

	   // Now loop through and sleep 1/10th second a max of 
	   // (10 * mean_time) times.  But break out "randomly".
   	for( int i = 0; i < intTenthsMax; i++ )
   	{
   		if ( rand() > dblPrnd ) break;
   		Sleep(100);
   	}
   }
	
	
//+------------------------------------------------------------------------------------+